classdef DemDiagnosticMonitor < autosar.bsw.ServiceImplementation
% Class to define Dem DiagnosticMonitor

%   Copyright 2016 The MathWorks, Inc.
    
    properties(Constant, Access=private)
        DefaultClientPortName = 'DiagnosticMonitor';
        DefaultInterfacePath = '/AUTOSAR/Services/Dem';
        InterfaceName = 'DiagnosticMonitor';
        Operations = {'SetEventStatus'
                      'ResetEventStatus'
                      'PrestoreFreezeFrame'
                      'ClearPrestoredFreezeFrame'
                      'SetEventDisabled'};

        FunctionPrototypes = {'ERR = %s_SetEventStatus(EventStatus)'
                            'ERR = %s_ResetEventStatus()'
                            'ERR = %s_PrestoreFreezeFrame()'
                            'ERR = %s_ClearPrestoredFreezeFrame()'
                            'ERR = %s_SetEventDisabled()'};

        InputArgSpecs = {'%s.getDefaultValue' % P_Dem_EventStatusType
                         ''
                         ''
                         ''
                         ''};
                         
        OutputArgSpecs = {'uint8(1)'
                          'uint8(1)'
                          'uint8(1)'
                          'uint8(1)'
                          'uint8(1)'};
                
        EnumDatatypes = {'Enum: Dem_EventStatusType'
                         ''
                         ''
                         ''
                         ''};

        DatatypeMaskPromptName = {'Data type for EventStatus:'
                                  ''
                                  ''
                                  ''
                                  ''};

        FunctionPrototypeMap = containers.Map( autosar.bsw.DemDiagnosticMonitor.Operations, ...
                                               autosar.bsw.DemDiagnosticMonitor.FunctionPrototypes );
                                           
        InputArgSpecMap = containers.Map( autosar.bsw.DemDiagnosticMonitor.Operations, ...
                                          autosar.bsw.DemDiagnosticMonitor.InputArgSpecs );

        OutputArgSpecMap = containers.Map( autosar.bsw.DemDiagnosticMonitor.Operations, ...
                                           autosar.bsw.DemDiagnosticMonitor.OutputArgSpecs );

        EnumDatatypeMap = containers.Map( autosar.bsw.DemDiagnosticMonitor.Operations, ...
                                          autosar.bsw.DemDiagnosticMonitor.EnumDatatypes );
        
        DatatypeMaskPromptNameMap = containers.Map( autosar.bsw.DemDiagnosticMonitor.Operations, ...
                                         autosar.bsw.DemDiagnosticMonitor.DatatypeMaskPromptName );

    end
    
    methods(Access=public)
        
        function this = DemDiagnosticMonitor()
        end
        
        function updateFunctionCaller(this, blkPath, portName, operationName)

            % Define Dem enumeration types
            autosar.bsw.Dem_defineIntEnumTypes;
            
            functionPrototypeTemplate = this.FunctionPrototypeMap(operationName);
            functionPrototype = sprintf(functionPrototypeTemplate, portName);
            
            inputArgSpec = this.InputArgSpecMap(operationName);
            outputArgSpec = this.OutputArgSpecMap(operationName);
            if strcmp(autosar.bsw.DemDiagnosticMonitor.getDatatypeVisibility(operationName), 'on')
                dataType = strrep(strtrim(get_param(blkPath, 'Datatype')), 'Enum: ', '');
                inputArgSpec = sprintf(inputArgSpec, dataType);
            end
            
            set_param(blkPath, 'FunctionPrototype', functionPrototype);
            set_param(blkPath, 'InputArgumentSpecifications', inputArgSpec);
            set_param(blkPath, 'OutputArgumentSpecifications', outputArgSpec);
            
        end

    end
    
    methods(Static, Access=public)

        function deepCopyInterface(mdlName, dstInterfacePath) %#ok<INUSD>
        % Copy interface and data types to model
        
            arxmlPath = fullfile(autosarroot, '+autosar', '+bsw', '+arxml');
            arxmlFiles = {fullfile(arxmlPath, 'AUTOSAR_BaseTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Type.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_MOD_PlatformTypes.arxml')
                          fullfile(arxmlPath, 'AUTOSAR_Dem.arxml')};
            
            autosar.bsw.ServiceImplementation.updateReferences(mdlName, arxmlFiles);            
        end
        
        function defaultClientPortName = getDefaultClientPortName()
            defaultClientPortName =  autosar.bsw.DemDiagnosticMonitor.DefaultClientPortName;
        end
       
        function dataType = getDefaultDatatype(defaultOperation)
            dataType = autosar.bsw.DemDiagnosticMonitor.EnumDatatypeMap(defaultOperation);
        end
        
        function interfaceName = getInterfaceName()
            interfaceName =  autosar.bsw.DemDiagnosticMonitor.InterfaceName;
        end
        
        function operations = getOperations()
            operations =  autosar.bsw.DemDiagnosticMonitor.Operations;
        end
        
        function hastypeParameter = hasDatatypeParameter()
            hastypeParameter = true;
        end
        
        function dataTypeCallback(blkPath)
            autosar.bsw.Dem_defineIntEnumTypes;
            operation = get_param(blkPath, 'Operation');
            if strcmp(autosar.bsw.DemDiagnosticMonitor.getDatatypeVisibility(operation), 'on')
                autosar.bsw.ServiceImplementation.dataTypeCallbackImpl(blkPath);
            end
        end
        
        function operationCallback(blkPath)
            operationValue = get_param(blkPath, 'Operation');
            autosar.bsw.ServiceImplementation.operationCallbackImpl(blkPath,...
                autosar.bsw.DemDiagnosticMonitor.EnumDatatypeMap(operationValue),...
                autosar.bsw.DemDiagnosticMonitor.EnumDatatypes,...
                autosar.bsw.DemDiagnosticMonitor.DatatypeMaskPromptNameMap)
        end
        
        % returns true if we should show the Datatype parameter for the
        % specified operation
        function visibility = getDatatypeVisibility(operation)
            datatype = autosar.bsw.DemDiagnosticMonitor.EnumDatatypeMap(operation);
            if isempty(datatype)
                visibility = 'off';
            else
                visibility = 'on';
            end
        end
        
        function desc = getDescription()
            desc = autosar.bsw.DemDiagnosticInfo.getDescription();
        end
        
        function type = getType()
            type = 'DiagnosticMonitorCaller';
        end
        
        function help = getHelp()
            help = 'helpview(fullfile(docroot,''ecoder'',''helptargets.map''),''autosar_dem_diagnosticmonitor_block'');';
        end
        
    end
    
end

% LocalWords:  Prestore Prestored bsw helptargets dem diagnosticmonitor
