classdef ConfigSetValidator < autosar.validation.PhasedValidator
% ConfigSetValidator

%   Copyright 2013-2014 The MathWorks, Inc.
    
    methods (Access=protected)
        
        function verifyInitial(this, hModel)
            this.verifyNotAcceleratorMode(hModel);
            this.verifyMultiInstanceMode(hModel);
            this.verifyConfigSet(hModel);
        end
        
    end
    
    methods (Static, Access=public)
        
        function cs = getActiveConfigSet(hModel)

            cs = getActiveConfigSet(hModel);
            if ~autosar.api.Utils.isMapped(hModel)
                ai = get_param(hModel, 'RTWFcnClass');
                if ~isempty(ai)
                    if ai.RightClickBuild
                        cs = getActiveConfigSet(bdroot(ai.SubsysBlockHdl));
                    end
                end
            end                 
        end
    
    end
    
    
    methods (Static, Access=private)

        function verifyNotAcceleratorMode(hModel)
            
            if strcmpi(get_param(hModel, 'SimulationMode'), 'accelerator')
                DAStudio.error('RTW:autosar:accelSimForbiddenForAUTOSAR')
            end
           
        end
                
        function verifyMultiInstanceMode(hModel)
            
            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            
            if strcmp(get_param(cs, 'CodeInterfacePackaging'), 'Reusable function')
                                                               
                if ~autosar.api.Utils.isMapped(hModel)
                    DAStudio.error('RTW:autosar:expectAutosarInterface');
                end
                
                if autosar.validation.ExportFcnValidator.isExportFcn(hModel)
                    msg = DAStudio.message('RTW:autosar:MultiInstanceERTCodeIsConfigured');
                    DAStudio.error('RTW:fcnClass:finish', msg);
                end
            
                if ~strcmp(get_param(cs, 'ERTFilePackagingFormat'), 'Modular')
                    DAStudio.error('RTW:autosar:MultiInstanceFilePackaging');
                end                
                
                if strcmp(get_param(cs, 'InlineParams'), 'off')
                    DAStudio.error('RTW:autosar:MultiInstanceNeedsInlineParams');
                end
                
            end                        
        end

        
        function verifyConfigSet(hModel)

            cs = autosar.validation.ConfigSetValidator.getActiveConfigSet(hModel);
            
            % Is it an appropriate target?
            isCompliant = strcmp(get_param(cs,'AutosarCompliant'), 'on');
            if ~isCompliant
                msg = DAStudio.message('RTW:autosar:nonAutosarCompliant');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            maxShortNameLength = get_param(cs, 'AutosarMaxShortNameLength');
            if (32 <= maxShortNameLength) && (maxShortNameLength <= 128)
                % fine
            else
                msg = DAStudio.message('RTW:autosar:invalidMaxShortNameLength', maxShortNameLength, 32, 128);
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if strcmp(get_param(cs, 'CombineOutputUpdateFcns'), 'off')
                msg = DAStudio.message('RTW:autosar:combineOutputUpdate');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if strcmp(get_param(cs, 'SupportContinuousTime'), 'on');
                msg = DAStudio.message('RTW:autosar:noContinuousTimeSupport');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if strcmp(get_param(cs, 'SupportNonInlinedSFcns'), 'on');
                msg = DAStudio.message('RTW:autosar:noNonInlinedSFcnsSupport');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if strcmp(get_param(cs, 'SupportNonFinite'), 'on');
                msg = DAStudio.message('RTW:autosar:noNonFiniteSupport');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if strcmp(get_param(cs, 'SupportComplex'), 'on');
                msg = DAStudio.message('RTW:autosar:noComplexSupport');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end

            if ~autosar.validation.ExportFcnValidator.isExportFcn(hModel) && ...
                    ~strcmp(get_param(cs, 'SampleTimeConstraint'), 'STIndependent') ...
                    && ~strcmp(get_param(cs, 'SolverMode'), 'SingleTasking') && ...
                    strcmp(get_param(cs, 'AutoInsertRateTranBlk'), 'on')
                msg = DAStudio.message('autosarstandard:validation:autoInsertRateTranBlk');
                DAStudio.error('RTW:fcnClass:finish', msg);
            end
            
            schemaVersionHandler = autosar.SchemaVersionHandler.instance();
            schemaVersionHandler.setSchemaVersion(get_param(cs, 'AutosarSchemaVersion'));
        end
            
    end

end
